<?php

show_admin_bar(false);

if ( ! function_exists( 'lq_store_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 */
	function lq_store_setup() {
		/*
		 * Make theme available for translation.
		 */
		load_theme_textdomain( 'lq_store', get_template_directory() . '/languages' );

		/*
		 * Let WordPress manage the document title.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 */
		add_theme_support( 'post-thumbnails' );
		add_image_size( 'post_737x280', 737, 280, true );
		add_image_size( 'post_275x340', 275, 340, true );

		// This theme uses wp_nav_menu() in one location.
		register_nav_menus( array(
			'menu-1' => esc_html__( 'Primary Menu Left', 'lq_store' ),
			'menu-2' => esc_html__( 'Secondary Menu Right', 'lq_store' ),
		) );

	}
endif;
add_action( 'after_setup_theme', 'lq_store_setup' );


/**
 * Enqueue scripts and styles.
 */
function lq_store_scripts() {
	
	/**
	 * Enqueue styles.
	 */
	wp_enqueue_style('google-fonts', 'https://fonts.googleapis.com/css?family=Karla:400,700|Playfair+Display|Sacramento', array(), false, 'all');
	wp_enqueue_style('adobe-fonts', 'https://use.typekit.net/lkd6iny.css', array(), false, 'all');
	wp_enqueue_style('adobe-fonts', 'https://use.typekit.net/gzo7swr.css', array(), false, 'all');
	wp_enqueue_style('plugins', get_template_directory_uri() . '/css/plugins.css', array(), false, 'all');
	wp_enqueue_style( 'lq_store-style', get_stylesheet_uri() );

	/**
	 * Enqueue scripts.
	 */
	wp_enqueue_script('jquery');
	wp_enqueue_script('plugins', get_template_directory_uri() . '/js/plugins.js', array(), false, true);
	$gmap_api = get_field('google_map_api_key', 'options');
	$googleapi = "https://maps.googleapis.com/maps/api/js?key=$gmap_api";
	wp_enqueue_script('gmap_api', $googleapi, array(), false, true);
	wp_enqueue_script('scripts', get_template_directory_uri() . '/js/scripts.js', array(), false, true);

	$map_icon = get_field('map_pin', 'options');
	$google_map = get_field('google_map');

	$data = array(
		'gmap_latitude' => $google_map['lat'],
		'gmap_longitude' => $google_map['lng'],
		'gmap_address' => $google_map['address'],
		'map_icon' => $map_icon,
	);
	wp_localize_script('scripts', 'ajax', $data);
}
add_action( 'wp_enqueue_scripts', 'lq_store_scripts' );


/**
 * Register and enqueue a custom stylesheet in the WordPress admin.
 */
function lq_store_scripts_admin() {
    wp_enqueue_style('icon-fonts', get_template_directory_uri() . '/css/icon_fonts.css', array(), false, 'all');
}
add_action( 'admin_enqueue_scripts', 'lq_store_scripts_admin' );

/**
 * Implement the Custom Header feature.
 */
require get_template_directory() . '/navwalker.php';


/**
 * ACF OPTIONS PAGE
 *
 */
if(function_exists('acf_add_options_page')) {
    acf_add_options_page();
}

/**
 * Dashboard google map api key support.
 */
add_filter('acf/settings/google_api_key', function () {
	$gmap_api = get_field('google_map_api_key', 'options');
    return $gmap_api;
});

/**
 * hex color to rgbd
 */
function hex2rgb($hex) {
   	$hex = str_replace("#", "", $hex);

   	if(strlen($hex) == 3) {
		$r = hexdec(substr($hex,0,1).substr($hex,0,1));
		$g = hexdec(substr($hex,1,1).substr($hex,1,1));
		$b = hexdec(substr($hex,2,1).substr($hex,2,1));
   	} else {
		$r = hexdec(substr($hex,0,2));
		$g = hexdec(substr($hex,2,2));
		$b = hexdec(substr($hex,4,2));
   	}
   	return 'rgba('.$r.', '.$g.', '.$b.', 0.8)';
}

/**
 * Get all page id
 *
 */ 
function getPageID() {
	global $post;
	$postid = $post->ID;
	if(is_home() && get_option('page_for_posts')) {
		$postid = get_option('page_for_posts');
	}

	return $postid;
}

/*
Customized header title 
*/

function customizedtitle($title){
	if(is_category() || is_author()){
		return get_the_archive_title();
	} else if( is_search()){
		return sprintf( esc_html__( 'Search Results for: %s', 'lq_store' ), '<strong>' . get_search_query() . '</strong>' ); 
	}
	return $title;
}

/**
 * Limit blog text
 *
 */
function Limit_Text($x, $length) {
	if(strlen($x)<=$length) {
		echo $x;
	} else {
		$y=substr($x,0,$length) . '...';
		echo $y;
	}
}

/**
 * ACF not working use jquery
 *
 */
function hide_editor_custom_js() {
    echo '<script type="text/javascript">
			jQuery(document).ready(function(){
				jQuery("#page_template").change( function() {
					jQuery("#_home_page_options").hide();
					jQuery("#postdivrich").show();
					switch( jQuery( this ).val() ) {
						case "t_home.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_about.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_contact.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_events.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_loyalty.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_products.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;

						case "t_order_pickup.php":
				          jQuery("#_home_page_options").show();
				          jQuery("#postdivrich").hide();

				        break;
					}
				}).change();
			});
		</script>';
}
add_action('admin_head', 'hide_editor_custom_js');


/**
 * Gravity form user role
 *
 */
function gforms_editor_access() {
  
  $role = get_role( 'editor' );
  $role->add_cap( 'gform_full_access' );
}
add_action( 'after_switch_theme', 'gforms_editor_access' );


/**
 * Numbered Pagination
 *
 */

if ( !function_exists( 'lq_store_pagination' ) ) {
	
	function lq_store_pagination() {

		global $paged;

		if(empty($paged)) $paged = 1;
	 
	    if($pages == '') {
	        global $wp_query;
	        $pages = $wp_query->max_num_pages;
	        if(!$pages) {
	            $pages = 1;
	        }

			echo '<div id="pagination"><span class="separator-bottom"></span><div class="row"><div class="col-md-9 col-sm-9 col-xs-8 col">';

			the_posts_pagination(array(
			   'prev_next' => true,
			   'prev_text' => '<i class="icon-arrow-left"></i>',
			   'next_text' => '<i class="icon-arrow-right"></i>',
			   'screen_reader_text' => '',
			   'before_page_number' => '',
			   'after_page_number' => '',
			   'show_all' => false,

			  )); 
			echo '</div>';

			echo '<div class="col-md-3 col-sm-3 col-xs-4 col text-right"><span>Page '. $paged .' of '. $pages .'</span>';

			echo '</div></div>';
			
		}
	}
	
}


/**
 * Gravity form anchor
 *
 */
add_filter( 'gform_confirmation_anchor', '__return_false' );


/**
 * Remove dashboard menu
 *
 */
function remove_admin_menus() {
	//remove_menu_page( 'themes.php' );               			// Appearance
	remove_menu_page( 'edit-comments.php' );      				// Comments
	//remove_menu_page( 'users.php' );                			// Users
	remove_menu_page( 'tools.php' );                			// Tools
	//remove_menu_page( 'options-general.php' );      			// Settings
	//remove_menu_page( 'plugins.php' );              			// Plugins
	//remove_menu_page( 'edit.php?post_type=acf-field-group' );	// Custom Field 
	remove_menu_page( 'pods' );               					// Pods Custom post type
}
add_action( 'admin_menu', 'remove_admin_menus', 999);


/**
 * Reorder dashboard menu
 *
 */
function reorder_admin_menu( $__return_true ) {
    return array(
         'index.php', 							// Dashboard
         'separator1', 							// --Space--
         'acf-options',							// ACF Theme Settings
         'edit.php?post_type=page', 			// Pages 
         'edit.php', 							// Posts
         'edit.php?post_type=event', 		// attorneys
         'separator2', 							// --Space--
         'gf_edit_forms',						// Gravity Forms
         'upload.php', 							// Media
         'wpseo_dashboard',						// Yoasta
         'gadash_settings',						// Google Analytics
         'themes.php', 							// Appearance
         'edit-comments.php', 					// Comments 
         'users.php', 							// Users
         'tools.php', 							// Tools
         'options-general.php', 				// Settings
         'plugins.php', 						// Plugins
   );
}
add_filter( 'custom_menu_order', 'reorder_admin_menu' );
add_filter( 'menu_order', 'reorder_admin_menu' );


/////////////////////////////////////////////////////////////////////////////////////////////////////

//GC Color Theme
function additional_admin_color_schemes() {
  //Get the theme directory
  $theme_dir = get_template_directory_uri();
 
  //GoingClear
 wp_admin_css_color(
   'goingclear',
   __('GoingClear'),
   admin_url("css/colors/goingclear/colors.css"),
   array('#8ec652', '#008cc6', '#e5e5e5', '#999'),
   array( 'base' => '#e5f8ff', 'focus' => '#fff', 'current' => '#fff' )
 );
}
add_action('admin_init', 'additional_admin_color_schemes');


//Reset GC Color Theme as Default for New Users
function set_default_admin_color($user_id) {
  $args = array(
    'ID' => $user_id,
    'admin_color' => 'goingclear'
  );
  wp_update_user( $args );
}
add_action('user_register', 'set_default_admin_color');


/////////////////////////////////////////////////////////////////////////////////////////////////////

//Remove Login Wiggle
function remove_login_shake() {
  // remove the wp_shake JavaScript
  remove_action( 'login_head', 'wp_shake_js', 12 );
}
add_action( 'login_head', 'remove_login_shake' );


function custom_header() { 
    echo '<link rel="stylesheet" type="text/css" href="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/login.css" />'; 
    echo '<script type="text/javascript" src="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/jquery-3.2.1.min.js"></script>';
    echo '<script type="text/javascript" src="' . get_bloginfo('template_directory') . '../../../../wp-admin/admin/login.js"></script>';
	 
}
add_action('login_head', 'custom_header');

/////////////////////////////////////////////////////////////////////////////////////////////////////
